@@ -335,6 +335,8 @@ NULL, /* istem_struct_base */
 0, /* istem_struct_base_siz_alloc */
 0, /* stem_valid  */
 0, /* istem_valid */
+NULL, /* fst_synclock_str */
+NULL, /* synclock_jrb */
 
 
 /*
@@ -1067,6 +1069,7 @@ NULL, /* sig_root_treesearch_gtk2_c_1 458 */
 NULL, /* sst_sig_root_treesearch_gtk2_c_1 */
 NULL, /* filter_str_treesearch_gtk2_c_1 459 */
 ND_DIR_UNSPECIFIED, /* filter_typ_treesearch_gtk2_c_1 */
+0, /* filter_typ_polarity_treesearch_gtk2_c_1 */
 0, /* filter_matlen_treesearch_gtk2_c_1 */
 0, /* filter_noregex_treesearch_gtk2_c_1 */
 #if defined(WAVE_USE_GTK2)
@@ -2024,6 +2027,12 @@ void reload_into_new_context_2(void)
  	GLOBALS->subvar_jrb_count = 0;
 	}
 
+ if(GLOBALS->synclock_jrb)
+	{
+ 	jrb_free_tree(GLOBALS->synclock_jrb);
+	GLOBALS->synclock_jrb = NULL;
+	}
+
 
  /* deallocate any loader-related stuff */
  switch(GLOBALS->loaded_file_type) {
