@@ -334,6 +334,8 @@ uint32_t istem_struct_base_siz;
 uint32_t istem_struct_base_siz_alloc;
 unsigned stem_valid : 1;
 unsigned istem_valid : 1;
+char *fst_synclock_str;
+JRB synclock_jrb;
 
 
 /*
@@ -1049,6 +1051,7 @@ struct tree *sig_root_treesearch_gtk2_c_1; /* from treesearch_gtk2.c 488 */
 struct tree *sst_sig_root_treesearch_gtk2_c_1; /* from treesearch_gtk2.c */
 char *filter_str_treesearch_gtk2_c_1; /* from treesearch_gtk2.c 489 */
 int filter_typ_treesearch_gtk2_c_1;
+int filter_typ_polarity_treesearch_gtk2_c_1;
 int filter_matlen_treesearch_gtk2_c_1;
 unsigned char filter_noregex_treesearch_gtk2_c_1;
 #if defined(WAVE_USE_GTK2)
